package jlk;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

public class FloatingDialog extends JDialog
   implements FloatingLicenseAddHostDelegate
{
   JPanel topPanel;
   JLabel titleLabel;
   JPanel midPanel;
   JScrollPane hostsScrollPane;
   JTextArea hostsTextArea;
   JPanel bottomPanel;
   JTextArea msgTextArea;
   JButton demoButton;
   JButton quitButton;
   EmptyBorder msgBorder;

   Frame parent;
   boolean cancelled = true;
   boolean setInitialFocus = false;
   boolean firstHostname = true;
   boolean showDemoButton = false;


   public FloatingDialog()
   {
      this((Frame)null, false);
   }

   public FloatingDialog(Frame aFrame, boolean modal)
   {
      super(aFrame, modal);
      parent = aFrame;

      topPanel = new JPanel();
      titleLabel = new JLabel();
      midPanel = new JPanel();
      hostsScrollPane = new JScrollPane();
      hostsTextArea = new JTextArea();
      bottomPanel = new JPanel();
      msgTextArea = new JTextArea();
      demoButton = new JButton();
      quitButton = new JButton();
      msgBorder = new EmptyBorder(0,0,0,0);

      setVisible(false);
      setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      getContentPane().setLayout(new BorderLayout(0,0));
      setSize(400, (modal) ? 230 : 160);

      topPanel.setLayout(new GridBagLayout());
      getContentPane().add("North", topPanel);
      titleLabel.setText(" Checking Network...");
      try {
         titleLabel.setIcon(new ImageIcon(getClass().getResource("appIcon.gif")));
      }
      catch(Exception e) { }
      topPanel.add(titleLabel, new GridBagConstraintsD(0,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.BOTH,new Insets(16,16,8,16),0,0));
      titleLabel.setFont(new Font("Dialog", Font.BOLD, 24));

      midPanel.setLayout(new GridBagLayout());
      getContentPane().add("Center", midPanel);
      hostsScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
      hostsScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
      hostsScrollPane.setOpaque(true);
      midPanel.add(hostsScrollPane, new GridBagConstraintsD(0,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.BOTH,new Insets(8,16,16,16),0,32));
      hostsTextArea.setLineWrap(true);
      hostsTextArea.setWrapStyleWord(true);
      hostsTextArea.setEditable(false);
      hostsScrollPane.getViewport().add(hostsTextArea);
      hostsTextArea.setFont(new Font("SansSerif", Font.BOLD|Font.ITALIC, 12));

      if (modal) {
         showDemoButton = LicenseHandler.isDemoModeEnabled();
         bottomPanel.setLayout(new GridBagLayout());
         getContentPane().add("South",bottomPanel);
         msgTextArea.setRows(2);
         msgTextArea.setText("All 5 available licenses are currently being used on the hosts listed above. Please try again later.");
         msgTextArea.setLineWrap(true);
         msgTextArea.setWrapStyleWord(true);
         msgTextArea.setOpaque(false);
         msgTextArea.setBorder(msgBorder);
         msgTextArea.setEditable(false);
         bottomPanel.add(msgTextArea, new GridBagConstraintsD(0,0,2,1,1.0,0.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.BOTH,new Insets(0,20,0,12),0,12));
         msgTextArea.setFont(new Font("SansSerif", Font.BOLD, 12));
         if (showDemoButton) {
            demoButton.setText("Demo");
            demoButton.setActionCommand("Demo");
            bottomPanel.add(demoButton, new GridBagConstraintsD(0,1,1,1,1.0,0.0,java.awt.GridBagConstraints.EAST,java.awt.GridBagConstraints.NONE,new Insets(8,0,8,2),12,0));
            quitButton.setText("Quit");
            quitButton.setActionCommand("Quit");
            bottomPanel.add(quitButton, new GridBagConstraintsD(1,1,1,1,1.0,0.0,java.awt.GridBagConstraints.WEST,java.awt.GridBagConstraints.NONE,new Insets(8,2,8,0),12,0));
         } else {
            quitButton.setText("Quit");
            quitButton.setActionCommand("Quit");
            bottomPanel.add(quitButton, new GridBagConstraintsD(0,1,2,1,0.0,0.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.NONE,new Insets(8,2,8,0),12,0));
         }
      }

      SymWindow aSymWindow = new SymWindow();
      this.addWindowListener(aSymWindow);

      if (modal) {
         SymAction lSymAction = new SymAction();
         if (showDemoButton)
            demoButton.addActionListener(lSymAction);
         quitButton.addActionListener(lSymAction);
         SymFocus aSymFocus = new SymFocus();
         hostsTextArea.addFocusListener(aSymFocus);
      }

      java.util.ResourceBundle b = LicenseHandler.getLicenseStrings();
      titleLabel.setText(b.getString("NETWORK_TITLE"));
      if (modal) {
         if (showDemoButton)
            demoButton.setText(b.getString("NETWORK_DEMO"));
         quitButton.setText(b.getString("NETWORK_QUIT"));
      }
   }

   public void runNonModal ()
   {
      pack();

      Dimension size = getSize();
      if (size.width < 400)
         setSize(size.width = 400, size.height);

      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (screenSize.width - size.width) / 2;
      int y = (screenSize.height - size.height) / 3;
      setLocation(x, y);

      firstHostname = true;
      hostsTextArea.setText("");

      setVisible(true);
   }

   private void interruptNonModal()
   {
      ResourceBundle b = LicenseHandler.getLicenseStrings();

      String[] options = {
         b.getString("NETWORK_NM_QUIT"),
         b.getString("NETWORK_NM_CONTINUE")
      };
      if (AlertDialog.runModalDialog(null, LicenseHandler.getAppName(),
         b.getString("NETWORK_NM_MSG"),
         AlertDialog.WARNING_MESSAGE,
         options) == 0) {
         try {
            LicenseHandlerExitDelegate ed = LicenseHandler.getExitDelegate();
            if (ed != null)
               ed.exitApplication();
         } catch (Exception e) {
         }
         System.exit(1);
      }
   }

   public void addHost (String aHostname)
   {
      if (firstHostname)
         firstHostname = false;
      else
         hostsTextArea.append(" ...");

      hostsTextArea.append(aHostname);

      show();
   }

   public void runModal (String msgText)
   {
      FloatingDialog mfd = new FloatingDialog(parent, true);

      mfd.setTitle(getTitle());
      mfd.setLocation(getLocation());

      cancelled = mfd.cancelled = true;

      mfd.msgTextArea.setText(msgText);

      mfd.pack();
      Dimension size = mfd.getSize();
      if (size.width < 400)
         mfd.setSize(size.width = 400, size.height);

      mfd.hostsTextArea.append(hostsTextArea.getText());

      mfd.setInitialFocus = true;

      setVisible(false);
      java.awt.Toolkit.getDefaultToolkit().beep();
      mfd.setVisible(true);

      cancelled = mfd.cancelled;

      mfd.setVisible(false);
      mfd.dispose();
   }

   public boolean wasCancelled ()
   {
      return cancelled;
   }

   protected JRootPane createRootPane()
   {
      escapeListener = new ActionListener() {
         public void actionPerformed(ActionEvent actionEvent) {
            if (isModal())
               setVisible(false);
            else
               interruptNonModal();
         }
      };
      JRootPane rootPane = new JRootPane();
      rootPane.registerKeyboardAction(escapeListener,
         KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
         JComponent.WHEN_IN_FOCUSED_WINDOW);
      return rootPane;
   }
   ActionListener escapeListener;

   public void addNotify()
   {
      // Record the size of the window prior to calling parents addNotify.
      Dimension size = getSize();

      super.addNotify();

      if (frameSizeAdjusted)
         return;
      frameSizeAdjusted = true;

      // Adjust size of frame according to the insets
      Insets insets = getInsets();
      setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height);
   }

   // Used by addNotify
   boolean frameSizeAdjusted = false;


   class SymAction implements java.awt.event.ActionListener
   {
      public void actionPerformed(java.awt.event.ActionEvent event)
      {
         Object object = event.getSource();
         if (object == demoButton)
            demoButton_actionPerformed(event);
         else if (object == quitButton)
            quitButton_actionPerformed(event);
      }
   }

   void demoButton_actionPerformed(java.awt.event.ActionEvent event)
   {
      setVisible(false);
      cancelled = false;
   }

   void quitButton_actionPerformed(java.awt.event.ActionEvent event)
   {
      setVisible(false);
   }

   class SymWindow extends java.awt.event.WindowAdapter
   {
      public void windowClosing(java.awt.event.WindowEvent event)
      {
         Object object = event.getSource();
         if (object == FloatingDialog.this)
            FloatingDialog_windowClosing(event);
      }

      public void windowActivated(java.awt.event.WindowEvent event)
      {
         Object object = event.getSource();
         if (object == FloatingDialog.this)
            FloatingDialog_windowActivated(event);
      }

      public void windowOpened(java.awt.event.WindowEvent event)
      {
         Object object = event.getSource();
         if (object == FloatingDialog.this)
            FloatingDialog_windowOpened(event);
      }
   }

   void FloatingDialog_windowOpened(java.awt.event.WindowEvent event)
   {
      if (setInitialFocus) {
         if (showDemoButton)
            demoButton.requestFocus();
         else
            quitButton.requestFocus();
         setInitialFocus = false;
      }
   }

   void FloatingDialog_windowActivated(java.awt.event.WindowEvent event)
   {
      if (setInitialFocus) {
         if (showDemoButton)
            demoButton.requestFocus();
         else
            quitButton.requestFocus();
         setInitialFocus = false;
      }
   }

   void FloatingDialog_windowClosing(java.awt.event.WindowEvent event)
   {
      if (isModal())
         setVisible(false);
      else
         interruptNonModal();
   }

   class SymFocus extends java.awt.event.FocusAdapter
   {
      public void focusGained(java.awt.event.FocusEvent event)
      {
         Object object = event.getSource();
         if (object == hostsTextArea)
            hostsTextArea_FocusGained(event);
      }
   }

   void hostsTextArea_FocusGained(java.awt.event.FocusEvent event)
   {
      if (showDemoButton)
         demoButton.requestFocus();
   }
}